/***************************************************************************
 *   Copyright (C) 2006 by Vincenzo Forchi`,,,   *
 *   tus@ken   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


// cumbia
#include <cumbiapool.h>
#include <cuserviceprovider.h>
#include <cumacros.h>
#include <quapps.h>
// cumbia

#include "gofbpm.h"
#include "bpmplotscaledraw.h"
#include "gofbpmplot.h"

#include <QPalette>
#include <QTimer>
#include <qregularexpression.h>

#define MIN_SIZE1 QSize(240,220)
#define MIN_SIZE2 QSize(670,630)

#define TAB_NORMAL 0
#define TAB_SPECTRUM 3
#define TAB_SLOW 1
#define TAB_FAST 2
#define TAB_TBT 4

#include <QApplication>
#include <QSplashScreen>
#include <qulabelbase.h>
#include <quplotoptions.h>
#include <quwatcher.h>
#include <quapplication.h>

gofbpm::gofbpm(QWidget *parent) : QWidget(parent) {
    // cumbia
    QuApplication *a = static_cast<QuApplication *>(QCoreApplication::instance());
    CuModuleLoader mloader(a->cumbiaPool(), a->fpool(), &m_log_impl);
    ui = new Ui::bpm;
    ui->setupUi(this);
    // mloader.modules() to get the list of loaded modules
    // cumbia

    QVBoxLayout *slowlo = new QVBoxLayout(ui->wSlow);
    foreach(const QString& n, QStringList() << "pX" << "pY" << "pSum") {
        GofBpmPlot *p = new GofBpmPlot(ui->wSlow, cu_pool, m_ctrl_factory_pool);
        p->setObjectName(n);
        slowlo->addWidget(p);
    }

    // QVBoxLayout *fastlo = new QVBoxLayout(ui->tabWidget->widget(Fast));
    QLayout *fastlo = ui->tabWidget->widget(Fast)->layout();
    foreach(const QString& n, QStringList() << "pXFast" << "pYFast" << "pSumFast") {
        GofBpmPlot *p = new GofBpmPlot(ui->tabWidget->widget(Fast), cu_pool, m_ctrl_factory_pool);
        p->setObjectName(n);
        fastlo->addWidget(p);
    }

    QVBoxLayout *fftlo = new QVBoxLayout(ui->wFFT);
    foreach(const QString& n, QStringList() << "pFFTX" << "pFFTY" << "pFFTSum") {
        GofBpmPlot *p = new GofBpmPlot(ui->wFFT, cu_pool, m_ctrl_factory_pool);
        p->setObjectName(n);
        fftlo->addWidget(p);
    }

    // QVBoxLayout *evtlo = new QVBoxLayout(ui->tabWidget->widget(Event));
    QLayout* evtlo = ui->tabWidget->widget(Event)->layout();
    foreach(const QString& n, QStringList() << "pEventH" << "pEventV" << "pEventSum") {
        GofBpmPlot *p = new GofBpmPlot(ui->tabWidget->widget(Event), cu_pool, m_ctrl_factory_pool);
        p->setObjectName(n);
        evtlo->addWidget(p);
    }

    QVBoxLayout *evtfftlo = new QVBoxLayout(ui->wEventFFT);
    foreach(const QString& n, QStringList() << "pEventFFTH" << "pEventFFTV" << "pEventFFTSum") {
        GofBpmPlot *p = new GofBpmPlot(ui->wEventFFT, cu_pool, m_ctrl_factory_pool);
        p->setObjectName(n);
        evtfftlo->addWidget(p);
    }

    QVBoxLayout *rawlo = new QVBoxLayout(ui->wRAW);
    GofBpmPlot *p = new GofBpmPlot(ui->wRAW, cu_pool, m_ctrl_factory_pool);
    p->setObjectName("pTBTX");
    rawlo->addWidget(p);

    //ui.setupUi(this);
    // ui->moreFrame->hide();
    // ui->tabWidget->hide();
    QTimer::singleShot(300, this, SLOT(init()));
}

void gofbpm::init()
{
    // periodically read Frequency and FastFrequency to adjust time axes
    // (sample frequency)
    member = ui->tStatus->source().section('/', -1);
    m_setBooleanDisplay(ui->tAGC, "AGC OFF", "AGC ON", QColor("orange"),
                                QColor("green"));
    m_setBooleanDisplay(ui->tSwitching, "Switching Off", "Switching On",
                                      QColor("orange"),
                                      QColor("green"));
    ui->tDSC->setEnumDisplay(0, "DSC off", QColor("orange"));
    ui->tDSC->setEnumDisplay(1, "DSC unity", QColor("orange"));
    ui->tDSC->setEnumDisplay(2, "DSC auto", QColor("green"));
    ui->tDSC->setEnumDisplay(3, "DSC save", QColor("orange"));
    ui->tMCPLL->setEnumDisplay(0, "MC Unlocked", QColor("red"));
    ui->tMCPLL->setEnumDisplay(1, "MC Locked", QColor("green"));
    m_setBooleanDisplay(ui->tMCPLL, "MC Unlocked", "MC Locked",
                                  QColor("red"),
                                  QColor("green"));
    ui->tSCPLL->setEnumDisplay(0, "SC Unlocked", QColor("yellow"));
    ui->tSCPLL->setEnumDisplay(1, "SC Locked", QColor("green"));
    m_setBooleanDisplay(ui->tSCPLL, "SC Unlocked",
                        "SC Locked",
                        QColor("orange"),
                        QColor("green"));
    // connect(ui->pushMore, SIGNAL(toggled(bool)), this, SLOT(showMore(bool)));
    connect(ui->tabWidget, SIGNAL(currentChanged(int)), this, SLOT(onTabChanged(int)));

    // ui->tInterlock->addSource("$1/InterlockConfiguration");

    setWindowTitle(member);

    QTimer::singleShot(600, this, SLOT(configurePlots()));
    QTimer::singleShot(610, this, SLOT(onTabChanged()));
    connect(ui->qioFastFFTSiz->outputWidget(), SIGNAL(newData(CuData)), findChild<GofBpmPlot *>("pFFTX"), SLOT(fftSizChanged(CuData)));
    connect(ui->qioFastFFTSiz->outputWidget(), SIGNAL(newData(CuData)), findChild<GofBpmPlot *>("pFFTY"), SLOT(fftSizChanged(CuData)));
    connect(ui->qioFastFFTSiz->outputWidget(), SIGNAL(newData(CuData)), findChild<GofBpmPlot *>("pFFTSum"), SLOT(fftSizChanged(CuData)));
    QuWatcher *w = new QuWatcher(this, cu_pool, m_ctrl_factory_pool);
    w->attach(this, SLOT(onNewFrequency(CuData)));
    w->setSource("$1/FastFrequency");
}

void gofbpm::m_setBooleanDisplay(QuLabelBase *l, const QString &fs, const QString &ts, const QColor &fc, const QColor &tc) {
    l->setProperty("falseString", fs); l->setProperty("trueString", ts);
    l->setProperty("falseColor", fc); l->setProperty("trueColor", tc);
}


void gofbpm::configurePlots() {
    // foreach(BpmPlot *plot, findChildren<BpmPlot *>()) {
    //     plot->setTitle("");
    //     QuPlotOptions o(plot);
    //     o.opengl(true);
    // }
    findChild<GofBpmPlot *>("pX")->setAxisTitle(QwtPlot::yLeft, "H [mm]");
    findChild<GofBpmPlot *>("pX")->setAxisTitle(QwtPlot::xBottom, "Time [s]");
    findChild<GofBpmPlot *>("pY")->setAxisTitle(QwtPlot::yLeft, "V [mm]");
    findChild<GofBpmPlot *>("pY")->setAxisTitle(QwtPlot::xBottom, "Time [s]");
    findChild<GofBpmPlot *>("pSum")->setAxisTitle(QwtPlot::yLeft, "Sum [au]");
    findChild<GofBpmPlot *>("pSum")->setAxisTitle(QwtPlot::xBottom, "Time [s]");
    // foreach(BpmPlot *p, QList<BpmPlot *>() << ui->pX << ui->pY << ui->pSum)
    //     p->setAxisScaleDraw(QwtPlot::xBottom, new BpmPlotScaleDraw);

    findChild<GofBpmPlot *>("pFFTX")->setAxisTitle(QwtPlot::yLeft, "H [mm]");
    findChild<GofBpmPlot *>("pFFTX")->setAxisTitle(QwtPlot::xBottom, "f [Hz]");

    findChild<GofBpmPlot *>("pFFTY")->setAxisTitle(QwtPlot::yLeft, "H [mm]");
    findChild<GofBpmPlot *>("pFFTY")->setAxisTitle(QwtPlot::xBottom, "f [Hz]");

    findChild<GofBpmPlot *>("pFFTSum")->setAxisTitle(QwtPlot::yLeft, "H [mm]");
    findChild<GofBpmPlot *>("pFFTSum")->setAxisTitle(QwtPlot::xBottom, "f [Hz]");

    findChild<GofBpmPlot *>("pXFast")->setAxisTitle(QwtPlot::yLeft, "H [mm]");
    findChild<GofBpmPlot *>("pYFast")->setAxisTitle(QwtPlot::yLeft, "V [mm]");
    findChild<GofBpmPlot *>("pSumFast")->setAxisTitle(QwtPlot::yLeft, "Sum [au]");

    // 4 checkboxes



    //ui->tPlotTBTY->addSource("$2/ZPosDD");
}


void gofbpm::onTabChanged(int t) {
    const CuData pd(CuDType::Period, 100);
    foreach (GofBpmPlot *p, findChildren<GofBpmPlot *>()) {
        p->unsetSources();
        p->setOptions(pd);
    }
    if(t == Pages::Slow) {
        findChild<GofBpmPlot *>("pX")->addSource("$1/HorPosHistory");
        findChild<GofBpmPlot *>("pY")->addSource("$1/VerPosHistory");
        findChild<GofBpmPlot *>("pSum")->addSource("$1/SumHistory");
    } else if(t == Pages::Fast) {
        findChild<GofBpmPlot *>("pXFast")->addSource(QString("$1/HorPosFastHistory"));
        findChild<GofBpmPlot *>("pYFast")->addSource(QString("$1/VerPosFastHistory"));
        findChild<GofBpmPlot *>("pSumFast")->addSource(QString("$1/SumFastHistory"));
    } else if(t == Pages::FastFFT) {
        findChild<GofBpmPlot *>("pFFTX")->addSource("$1/HorPosFastFFT");
        findChild<GofBpmPlot *>("pFFTY")->addSource("$1/VerPosFastFFT");
        findChild<GofBpmPlot *>("pFFTSum")->addSource("$1/SumFastFFT");
    }
    else if(t == Pages::Event) {
        findChild<GofBpmPlot *>("pEventH")->addSource("$1/HorPosEventHistory");
        findChild<GofBpmPlot *>("pEventV")->addSource("$1/HorPosEventHistory");
        findChild<GofBpmPlot *>("pEventSum")->addSource("$1/HorPosEventHistory");
    } else if(t == Pages::EventFFT) {
        findChild<GofBpmPlot *>("pEventFFTH")->addSource("$1/HorPosEventFFT");
        findChild<GofBpmPlot *>("pEventFFTV")->addSource("$1/HorPosEventFFT");
        findChild<GofBpmPlot *>("pEventFFTSum")->addSource("$1/HorPosEventFFT");

    } else if( t == Pages::RAW) {
        findChild<GofBpmPlot *>("pTBTX")->addSource("$1/VaVbVcVdRaw");
    }
}

void gofbpm::onNewFrequency(const CuData &da) {
    float fastf = 1.0;
    if(!da.B(CuDType::Err) && da.containsKey(CuDType::Value)) {
        da[CuDType::Value].to<float>(fastf);
        findChild<GofBpmPlot *>("pFFTX")->fastFreqChanged(fastf);
        findChild<GofBpmPlot *>("pFFTY")->fastFreqChanged(fastf);
        findChild<GofBpmPlot *>("pFFTSum")->fastFreqChanged(fastf);
    }
}
